class PlpygisError(Exception):
    """
    Basic exception for ``plpygis``.
    """
    def __init__(self, msg):
        super(PlpygisError, self).__init__(msg)
      

class DependencyError(PlpygisError, ImportError):
    """
    Exception for a missing dependency.
    """
    def __init__(self, dep, msg=None):
        if msg is None:
            msg = "Dependency '{}' is not available.".format(dep)
        super(DependencyError, self).__init__(msg)


class WkbError(PlpygisError):
    """
    Exception for problems in parsing WKBs.
    """
    def __init__(self, msg=None):
        if msg is None:
            msg = "Unreadable WKB."
        super(WkbError, self).__init__(msg)


class DimensionalityError(PlpygisError):
    """
    Exception for problems in dimensionality of geometries.
    """
    def __init__(self, msg=None):
        if msg is None:
            msg = "Geometry has invalid dimensionality."
        super(DimensionalityError, self).__init__(msg)


class SridError(PlpygisError):
    """
    Exception for problems in dimensionality of geometries.
    """
    def __init__(self, msg=None):
        if msg is None:
            msg = "Geometry has invalid SRID."
        super(SridError, self).__init__(msg)



from binascii import hexlify, unhexlify
from struct import calcsize, unpack_from, pack


class HexReader:
    """
    A reader for generic hex data. The current position in the stream of bytes
    will be retained as data is read.
    """

    def __init__(self, hexdata, order, offset=0):
        self._data = hexdata
        self._order = order
        self._ini_offset = offset
        self._cur_offset = offset

    def reset(self):
        """
        Start reading from the initial position again.

        :rtype: int
        """
        self._cur_offset = self._ini_offset

    def _get_value(self, fmt):
        value = unpack_from("{}{}".format(self._order, fmt), self._data, self._cur_offset)[0]
        self._cur_offset += calcsize(fmt)
        return value

    def get_char(self):
        return self._get_value("B")

    def get_int(self):
        """
        Get the next four-byte integer from the stream of bytes.

        :rtype: int
        """
        return self._get_value("I")

    def get_double(self):
        """
        Get the next double from the stream of bytes.

        :rtype: int
        """
        return self._get_value("d")


class HexWriter:
    def __init__(self, order):
        self._fmts = []
        self._values = []
        self._order = order

    def _add_value(self, fmt, value):
        self._fmts.append(fmt)
        self._values.append(value)

    def add_order(self):
        if self._order == "<":
            self.add_char(1)
        else:
            self.add_char(0)

    def add_char(self, value):
        self._add_value("B", value)

    def add_int(self, value):
        self._add_value("I", value)

    def add_double(self, value):
        self._add_value("d", value)

    @property
    def data(self):
        fmt = self._order + "".join(self._fmts)
        data = pack(fmt, *self._values)
        return HexBytes(data)


class HexBytes(bytes):
    """A subclass of bytearray that represents binary WKB data.
    It can be converted to a hexadecimal representation of the data using str()
    and compared to a hexadecimal representation with the normal equality operator."""

    def __new__(cls, data):
        if not isinstance(data, (bytes, bytearray)):
            data = unhexlify(str(data))
        elif data[:2] in (b'00', b'01'):  # hex-encoded string as bytes
            data = unhexlify(data.decode('ascii'))
        return bytes.__new__(cls, data)

    def __str__(self):
        return hexlify(self).decode('ascii')

    def __eq__(self, other):
        if isinstance(other, str) and other[:2] in ('00', '01'):
            other = unhexlify(other)
        return super(HexBytes, self).__eq__(other)



import os
from setuptools import setup
from plpygis._version import __version__

def readme():
    with open('README.rst') as f:
        return f.read()

setup(
    name="plpygis",
    version=__version__,
    url="https://github.com/bosth/plpygis",
    license="GNU GPLv3",
    author="Benjamin Trigona-Harany",
    author_email="bosth@alumni.sfu.ca",
    description="PostGIS Python tools",
    long_description=readme(),
    packages=["plpygis"],
    include_package_data=True,
    platforms="any",
    classifiers = [
        "Topic :: Database",
        "Topic :: Scientific/Engineering :: GIS"
    ],
    extras_require = {
        "shapely_support":  ["Shapely>=1.5.0"]
    },
    keywords=["gis geospatial postgis postgresql plpython"]
)
